# coding: utf-8
'''
Helper functions for Intervene
'''
import sys
import os
import intervene
from matplotlib import colors

def create_dir(dir_path):
    '''
    Create a output directory if it's not exists.
    '''

    if not os.path.exists(dir_path):
        try:
            os.makedirs(dir_path)
        except:
            sys.exit( "Output directory (%s) could not be created." % dir_path )
    return dir_path

def venn_order(input_files):
    """
    Checks the order of venn plot. 

    @type input_files: list[Iterable] 
    @rtype int 

    input
      input_files: a list of files of genomic regions or lists

    return
      len(input_files): return length of input files
    """

    return len(input_files)

def get_filenames(input_files):
    """
    Checks the venn-type

    @type input_files: list[Iterable] 
    @rtype file_names: list[Iterable]

    input
      input_files: a list of files of genomic regions or lists

    return
      file_names:: return a list of file names
    """
    file_names=[]
    for fname in input_files:
      file_names.append(os.path.splitext(os.path.basename(fname))[0])

    return file_names

"""
Provides access to example files

"""

def data_dir():
    """
    Returns the data directory that contains example files.
    """
    #data_path = os.path.dirname(intervene.__file__)
    #data_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'example_data')
    #print(data_path)
    return os.path.join(os.path.dirname(__file__), 'example_data')


def example_filename(fn,sub_dir=None):
    """
    Return a bed file from the pybedtools examples directory.
    This code is adapted from https://github.com/daler/pybedtools

    """
    #print(data_dir())
    #sys.exit()
    if sub_dir:
      fn = os.path.join(data_dir(), sub_dir, fn)
    else:
      fn = os.path.join(data_dir(), fn)
    #print(fn)
    if not os.path.exists(fn):
        raise ValueError("%s does not exist" % fn)
    return fn

def get_test_data(module_name):
  '''
  It returns a list of test data files, if the user uses --test argument
  
  @type module_name: chr
  @rtype test_files: list[Iterable]

  input
      module_name: Name of Intervene module

  return
      test_files:: return a list of file paths

  '''

  test_files = []
  
  if module_name == 'venn':
    test_files = [example_filename(i,'ENCODE_hESC') for i in  [
        'H3K27ac.bed',
        #'H3K4me2.bed',
        'H3K4me3.bed',
        'H3K27me3.bed'
        ]]

  elif module_name == 'upset':
    test_files = [example_filename(i, 'ENCODE_hESC') for i in  [
        'H3K27ac.bed',
        #'H3K4me2.bed',
        #'H3K4me1.bed',
        'H3K4me3.bed',
        'H3K27me3.bed'
        ]]

  elif module_name == 'pairwise':
    test_files = [example_filename(i, 'dbSUPER_mm9') for i in  [
        'Bone_Marrow.bed',
        'Cerebellum.bed',
        'Cortex.bed',
        'E14.5_Brain.bed',
        'E14.5_Heart.bed',
        'E14.5_Limb.bed',
        'E14.5_Liver.bed',
        'Heart.bed',
        'HFSCs.bed',
        'Intestine.bed',
        'Kidney.bed',
        'Liver.bed',
        'Lung.bed',
        'Macrophage.bed',
        'MEF.bed',
        'mESC.bed',
        'Myotubes.bed',
        'Olfactory_Bulb.bed',
        'pro-B.bed',
        'Spleen.bed',
        'TACs.bed',
        'Testis.bed',
        'Th_Cells.bed',
        'Thymus.bed'
        ]]
  else:
    sys.exit(1)

  return test_files

def default_colors():
    """
    Get a list of RGBA colors 
    """
    # default_colors = [
    #     # r, g, b, a
    #     [92, 192, 98, 0.5],
    #     [90, 155, 212, 0.5],
    #     [246, 236, 86, 0.6],
    #     [241, 90, 96, 0.4],
    #     [255, 117, 0, 0.3],
    #     [82, 82, 190, 0.2],
    # ]

    default_colors = [
        # r, g, b, a
        [188, 114, 3, 0.5],
        [3, 133, 188, 0.5],
        [155, 9, 118, 0.6],
        [155, 53, 9, 0.4],
        [4, 140, 128, 0.3],
        [140, 8, 8, 0.2],
    ]

    default_colors = [
        [i[0] / 255.0, i[1] / 255.0, i[2] / 255.0, i[3]]
        for i in default_colors
    ]

    return default_colors

def get_colors(color_list):
    """
    Get color combinations for Venn diagram. This converts color names to RGBA 

    """
    rgba_colors = []
    a = [0.5,0.5,0.6,0.4,0.3,0.2]
    i = 0
    for c in color_list:
        rgba_colors.append(list(colors.to_rgba(c)))
        rgba_colors[i][3] = a[i]
        i+=1

    return rgba_colors



# coding: utf-8

"""
InterVene: a tool for intersection and visualization of multiple genomic region sets
Created on January 10, 2017
@author: <Aziz Khan>aziz.khan@ncmm.uio.no
"""
import sys
import os
from pybedtools import BedTool, helpers
from intervene.modules.venn import list_venn
from intervene.modules.upset import upset


def venn2(input_files, options,names=['A','B'], plot_type='venn'):

    a = BedTool(input_files[0])
    b = BedTool(input_files[1])


    labels = {'10': (a - b).count(), #Only A
     '01': (b - a).count(), #Only B
     '11': (a + b).count() #Common in A and B
     } 

    #delete all temp files
    helpers.cleanup()

    if plot_type == 'upset':
        upset.create_r_script(labels, names, options)
    
    else:
        fig, ax = list_venn.venn2(labels, names=names, dpi=options.dpi, colors=options.colors, figsize=options.figsize)

        return fig, ax

def venn3(input_files, options, names=['A','B','C'], plot_type='venn'):

    a = BedTool(input_files[0])
    b = BedTool(input_files[1])
    c = BedTool(input_files[2])

    
    labels = {'001': (c - a - b).count(), #Only C
    '010': (b - a - c).count(), #only
    '011': (b + c - a).count(),
    '100': (a - b - c).count(), #only A
    '101': (a + c - b).count(),
    '110': (a + b - c).count(),
    '111': (a + b + c).count()}

    #delete all temp files
    helpers.cleanup()

    if plot_type == 'upset':
        upset.create_r_script(labels, names, options)
        
    else:
        fig, ax = list_venn.venn3(labels, names=names, dpi=options.dpi, colors=options.colors,  figsize=options.figsize)
        return fig, ax


def venn4(input_files, options, names=['A','B','C','D'],plot_type='venn'):

    a = BedTool(input_files[0])
    b = BedTool(input_files[1])
    c = BedTool(input_files[2])
    d = BedTool(input_files[3])

   
    #ABCD
    labels = {'0001': str((d - a - b - c).count()),
    '0010': str((c - a - b - d).count()),
    '0011': str((d + c - b - a).count()),
    '0100': str((b - a - d - c).count()),
    '0101': str((b + d - a - c).count()),
    '0110': str((b + c - a - d).count()),
    '0111': str((d + b + c - a).count()),
    '1000': str((a - b - c - d).count()),
    '1001': str((a + d - b - c).count()),
    '1010': str((a + c - b - d).count()),
    '1011': str((a + c + d - b).count()),
    '1100': str((a + b - c - d).count()),
    '1101': str((a + b + d - c).count()),
    '1110': str((a + b + c - d).count()),
    '1111': str((a + b + c + d).count())
    }

    #delete all temp files
    helpers.cleanup()
    
    if plot_type == 'upset':
        upset.create_r_script(labels, names, options)
 
    else:
        fig, ax = list_venn.venn4(labels, names=names, dpi=options.dpi, colors=options.colors, figsize=options.figsize)

        return fig, ax

def venn5(input_files, options, names=['A','B','C','D','E'], plot_type='venn'):

    a = BedTool(input_files[0])
    b = BedTool(input_files[1])
    c = BedTool(input_files[2])
    d = BedTool(input_files[3])
    e = BedTool(input_files[4])


    #ABCDE
    labels = {'00001': str((e - a - b - c - d).count()),
    '00010': str((d - a - b - c - e).count()),
    '00011': str((d + e - a - b - c).count()),
    '00100': str((c - a - b - d - e).count()),
    '00101': str((c + e - a - b - d).count()),
    '00110': str((c + d - a - b - e).count()),
    '00111': str((c + d + e- a - b).count()),
    '01000': str((b - a - c - d - e).count()),
    '01001': str((b + e - a - c - d).count()),
    '01010': str((b + d - a - c - e).count()),
    '01011': str((b + d + e - a - c).count()),
    '01100': str((b + c - a - d - e).count()),
    '01101': str((b + c + e - a - d).count()),
    '01110': str((b + c + d - a - e).count()),
    '01111': str((b + c + d + e - a).count()),
    '10000': str((a - b - c - d - e).count()),
    '10001': str((a + e - b - c - d).count()),
    '10010': str((a + d - b - c - e).count()),
    '10011': str((a + d + e - b - c).count()),
    '10100': str((a + c - b - d - e).count()),
    '10101': str((a + c + e - b - d).count()),
    '10110': str((a + c + d - b - e).count()),
    '10111': str((a + c + d + e - b).count()),
    '11000': str((a + b - c - d - e).count()),
    '11001': str((a + b + e - c - d).count()),
    '11010': str((a + b + d - c - e).count()),
    '11011': str((a + b + d + e - c).count()),
    '11100': str((a + b + c - d - e).count()),
    '11101': str((a + b + c + e - d).count()),
    '11110': str((a + b + c + d - e).count()),
    '11111': str((a + b + c + d + e).count())
    }

    #delete all temp files
    helpers.cleanup()
    
    if plot_type == 'upset':
        #upset.draw_genomic(labels, names, output, fig_type, options)
        upset.create_r_script(labels, names, options)

    else:
        fig, ax = list_venn.venn5(labels, names=names, dpi=options.dpi, colors=options.colors, figsize=options.figsize)
        return fig, ax


def venn6(input_files, options, names=['A','B','C','D','E','F'],plot_type='venn'):
    """
    6-way Venn diagram from a list of six genomic region sets in <BED/GTF/GFF/VCF> format.

    """

    a = BedTool(input_files[0])
    b = BedTool(input_files[1])
    c = BedTool(input_files[2])
    d = BedTool(input_files[3])
    e = BedTool(input_files[4])
    f = BedTool(input_files[5])
   
    #ABCDEF
    labels = {'000001': str((f - a - b - c - d - e).count()),
    '000010': str((e - a - b - c - d - f).count()),
    '000011': str((e + f - a - b - c - d).count()),
    '000100': str((d - a - b - c - e - f).count()),
    '000101': str((d + f - a - b - c - e).count()),
    '000110': str((d + e - a - b - c - f).count()),
    '000111': str((d - e - f - a - b - c).count()),
    '001000': str((c - a - b - d - e - f).count()),
    '001001': str((c + f - a - b - d - e).count()),
    '001010': str((c + e - a - b - d - f).count()),
    '001011': str((c + e + f - a - b - d).count()),
    '001100': str((c + d - a - b - e - f).count()),
    '001101': str((c + d + f - a - b - e).count()),
    '001110': str((c + d + e - a - b - f).count()),
    '001111': str((c + d + e + f - a - b).count()),
    '010000': str((b - a - c - d - e - f).count()),
    '010001': str((b + f - a - c - d - e).count()),
    '010010': str((b + e - a - c - d - f).count()),
    '010011': str((b + e + f - a - c - d).count()),

    '010100': str((b + d - a - c - e - f).count()),
    '010101': str((b + d + f - a - c - e).count()),
    '010110': str((b + c + d + e + f - a).count()),
    '010111': str((b + d + e + f - a - c).count()),
    '011000': str((b + c - a - d - e - f).count()),
    '011001': str((b + c + f - a - d - e).count()),
    '011010': str((b + c + e - a - d - f).count()),
    '011011': str((b + c + e + f - a - d).count()),
    '011100': str((b + c + d - a - e - f).count()),
    '011101': str((b + c + d + f - a - e).count()),
    '011110': str((b + c + d + e - a - f).count()),

    '011111': str((b + c + d + e + f - a).count()),
    '100000': str((a - b - c - d - e - f).count()),
    '100001': str((a + f - b - c - d - e).count()),
    '100010': str((a + e - b - c - d - f).count()),
    '100011': str((a + e + f - b - c - d).count()),
    '100100': str((a + d - b - c - e - f).count()),
    '100101': str((a + d + f - b - c - e).count()),
    '100110': str((a + d + e - b - c - f).count()),
    '100111': str((a + d + e + f - b - c).count()),
    '101000': str((a + c - b - d - e - f).count()),
    '101001': str((a + c + f - d - c - e).count()),
    '101010': str((a + c + e - b - d - f).count()),
    '101011': str((a + c + e + f - b - d).count()),

    '101100': str((a + c + d - b - e - f).count()),
    '101101': str((a + c + d + f - b - e).count()),
    '101110': str((a + c + d + e - b - f).count()),
    '101111': str((a + c + d + e + f - b).count()),
    '110000': str((a + b - c - d - e - f).count()),
    '110001': str((a + b + f - c - d - e).count()),
    '110010': str((a + b + e - c - d - f).count()),
    '110011': str((a + b + e + f - c - d).count()),
    '110100': str((a + b + d - c - e - f).count()),
    '110101': str((a + b + d + f + e - c).count()),
    '110110': str((a + b + d + e - f - c).count()),
    '110111': str((a + b + d + e + f - c).count()),
    '111000': str((a + b + c - d - e - f).count()),
    '111001': str((a + b + c + f - d - e).count()),
    '111010': str((a + b + c + e - d - f).count()),
    '111011': str((a + b + c + e + f - d).count()),
    '111100': str((a + b + c + d - e - f).count()),
    '111101': str((a + b + c + d + f - e).count()),
    '111110': str((a + b + c + d + e - f).count()),
    '111111': str((a + b + c + d + e + f).count())
    }

    #delete all temp files
    helpers.cleanup()

    if plot_type == 'upset':
        upset.create_r_script(labels, names, options)

    else:
        fig, ax = list_venn.venn6(labels, names=names, dpi=options.dpi, colors=options.colors, figsize=options.figsize)
        
        return fig, ax



# coding: utf-8

"""
InterVene: a tool for intersection and visualization of multiple genomic region sets
Created on January 10, 2017
@author: <Aziz Khan>aziz.khan@ncmm.uio.no
"""
import sys
import os
import tempfile
import itertools
from intervene.modules.pairwise.pairwise import get_name
from pybedtools import BedTool, helpers


def genomic_upset(input_files):
    '''
    Arguments:
        input_files -  List of BED files to to calculate the weights
    Takes a list of sets a list of the sizes of non-overlapping intersections between them 
    '''
    
    N = len(input_files)
    
    # Generate a truth table of intersections to calculate 
    truth_table = [x for x in itertools.product("01", repeat=N)][1:]

    weights = {}

    for t in truth_table:
        ones = [BedTool(input_files[i]) for i in range(N) if t[i] =='1']
        zeros = [BedTool(input_files[i]) for i in range(N) if t[i] =='0']
        
        #report those entries in set A which do ovelap with other sets
        x = ones[0]
        if len(ones) > 1:
            for bed in ones[1:]:
                x = x.intersect(bed, u=True)

        #report those entries in set A which doesn't ovelap with other sets
        if len(zeros) > 0:
            #y = zeros[0]
            for bed in zeros[0:]:
                x = x.intersect(bed, v=True)
        
        X = (x).count() 

        weights[''.join(t)] = X

    #delete all temp files
    helpers.cleanup()
    
    return(weights)

def list_upset(input_files):
    '''
    Arguments:
        input_files -  List of list files to calculate weights for upset plot
    Takes a list of sets a list of the sizes of non-overlapping intersections between them 
    '''
    S =[]
    for f in input_files:
        a = open(f, 'r').read().splitlines()
        S.append(set(a))

    N = len(S)
    
    # Generate a truth table of intersections to calculate 
    truth_table = [x for x in itertools.product("01", repeat=N)][1:]

    
    weights = {}
    for t in truth_table:
        ones = [S[i] for i in range(N) if t[i] =='1']
        zeros = [S[i] for i in range(N) if t[i] =='0']
        
        X = set.intersection(*ones)
        X.difference_update(*zeros)
        
        weights[''.join(t)] = len(X)
    
    return(weights)

def create_r_script(labels, names, options):
    """
    It create Rscript for UpSetR plot for the genomic regions.

    """
    #temp_f = tempfile.NamedTemporaryFile(delete=False)
    #temp_f = open(tempfile.mktemp(), "w")
    script_file =  options.output+'/'+str(options.project)+'_'+options.command+'.R'
    temp_f = open(script_file, 'w')
    output_name = options.output+'/'+str(options.project)+'_'+options.command+'.'+options.figtype

    temp_f.write('#!/usr/bin/env Rscript'+"\n")
    temp_f.write('if (suppressMessages(!require("UpSetR"))) suppressMessages(install.packages("UpSetR", repos="http://cran.us.r-project.org"))\n')
    temp_f.write('library("UpSetR")\n')
    if options.figtype == 'ps':
        temp_f.write('if (suppressMessages(!require("Cairo"))) suppressMessages(install.packages("Cairo", repos="http://cran.us.r-project.org"))\n')
        temp_f.write('library("Cairo")\n')
    
    if options.figtype == 'pdf' or options.figtype == 'svg':
        temp_f.write(options.figtype+'("'+output_name+'", width='+str(options.figsize[0])+', height='+str(options.figsize[1])+', onefile=FALSE)'+'\n')
    
    elif options.figtype == 'ps':
        temp_f.write('cairo_ps("'+output_name+'", width='+str(options.figsize[0])+', height='+str(options.figsize[1])+')'+'\n')
    else:
        temp_f.write(options.figtype+'("'+output_name+'", width='+str(options.dpi*options.figsize[0])+', height='+str(options.dpi*options.figsize[1])+', res='+str(options.dpi)+')\n')
     
    temp_f.write("expressionInput <- c(")

    last = 1

    shiny = ""

    for key, value in labels.items(): #iteritems in python 2.7
        i = 0
        first = 1
        for x in key:
            if i == 0:
                temp_f.write("'")
      
            if x == '1':
                if first == 1:
                    temp_f.write(str(names[i]))
                    shiny += str(names[i])
                    first = 0
                else:
                    temp_f.write('&'+str(names[i]))
                    shiny += '&'+str(names[i])

            if i == len(key)-1:
                if last == len(labels):
                    temp_f.write("'="+str(value))
                    shiny += "="+str(value)

                else:
                    temp_f.write("'="+str(value)+',')
                    shiny += "="+str(value)+','
            i += 1
        last +=1
    temp_f.write(")\n")

    #options.shiny = True
    #If shiny output
    if options.showshiny == False:

        shiny_import =  options.output+'/'+str(options.project)+'_'+options.command+'_combinations.txt'
        shiny_file = open(shiny_import, 'w')
        shiny_file.write("You can go to Intervene Shiny App https://asntech.shinyapps.io/Intervene-app/ and copy/paste the following intersection data to get more interactive figures.\n\n")
        shiny_file.write(shiny)
        shiny_file.close()
    
    else:
        print(shiny)

    if options.showsize:
        options.showsize = 'yes'

    #if options.ninter == 0:
    #    options.ninter = "NA"

    if options.showzero == False:
        options.showzero = 'NULL'
    else:
        options.showzero = "'on'"

    temp_f.write('upset(fromExpression(expressionInput), nsets='+str(len(key))+', nintersects='+str(options.ninter)+', show.numbers="'+str(options.showsize)+'", main.bar.color="'+options.mbcolor+'", sets.bar.color="'+options.sbcolor+'", empty.intersections='+str(options.showzero)+', order.by = "'+options.order+'", number.angles = 0, mainbar.y.label ="'+options.mblabel+'", sets.x.label ="'+options.sxlabel+'")\n')
    temp_f.write('invisible(dev.off())\n')

    #print temp_f.read()
    #print temp_f.name
    #cmd = 'intervene_upset_plot.R %s %s %s' % ('genomic',5,temp_f.name)
    cmd = temp_f.name
    temp_f.close()

    if options.scriptonly == False:
        os.system('chmod +x '+cmd)
        os.system(cmd)
        print('\nYou are done! Please check your results @ '+options.output+'. \nThank you for using Intervene!\n')
        sys.exit(0)
    else:
        print('\nYou are done! Please check your UpSet plot script and Shiny App input @ '+options.output+'. \nThank you for using Intervene!\n')
        sys.exit(0)

        
def draw_genomic(labels, names, output, fig_type):
    #temp_f = tempfile.NamedTemporaryFile(delete=False)
    temp_f = open(tempfile.mktemp(), "w")
    
    temp_f.write("expressionInput <- c(")
    last = 1
    for key, value in labels.items():
        i = 0
        first = 1
        for x in key:
            if i == 0:
                #print("'")
                temp_f.write("'")      
            if x == '1':
                if first == 1:
                    temp_f.write(str(names[i]))
                    #print(str(names[i]))
                    first = 0
                else:
                    temp_f.write('&'+str(names[i]))
                    #print('&'+str(names[i]))

            if i == len(key)-1:
                if last == len(labels):
                    temp_f.write("'="+str(value))
                else:
                    temp_f.write("'="+str(value)+',')
                #print("'="+str(value)+',')
            i += 1
        last +=1
        #print("'="+str(value)+',')
        #temp_f.write("'="+str(value)+',')
    temp_f.write(")\n")
    #print temp_f.read()
    #print temp_f.name
    temp_f.close()
    cmd = 'upset_plot_intervene.R %s %s %s %s %s ' % ('genomic',len(key),temp_f.name, output, fig_type)
    os.system(cmd)
    sys.exit(1)


def one_vs_rest_intersection(beds, peaks, output, **kwoptions):
    '''
    Compares a set of peaks with several other peaks sets.

    '''
    names = []
    matrix_file = output+'/One_vs_all_peak_set_matrix.txt'
    f = open(matrix_file, 'w')
    
    f.write('peak_id')
    #f.write('peak_id\tchrom\tstart\tend')

    for bed in beds:
        names.append(get_name(bed))
        f.write('\t' + str(get_name(bed)))
    #main_int.append(names)

    peaks = BedTool(peaks[0])
    f.write('\n')
    for i in peaks:
        #region_int = []
        peak_id = str(i.chrom)+"_"+str(i.start)+"_"+str(i.end)
        f.write(peak_id)
        #f.write(peak_id + '\t' + i.chrom + '\t' + str(i.start) + '\t' + str(i.end))

        for bed in beds:
            b = BedTool(bed)
            int_count = BedTool(str(i), from_string=True).intersect(b).count()
            if (int_count > 0):
                #region_int.append("1")
                f.write('\t' + str(1))
            else:
                #region_int.append("0")
                f.write('\t' + str(0))
        f.write('\n')
        #main_int.append(region_int)
        #matrix[peak_id] = region_int
    f.close()

    return matrix_file
    


#!/usr/bin/env python

"""
This is a setup script for Intervene: a toolfor intersection and visualization of multiple genomic region sets

This code is free software; you can redistribute it and/or modify it under the terms of the 
BSD License (see the file LICENSE.md included with the distribution).

@author: Aziz Khan
@email: aziz.khan@ncmm.uio.no
"""
import os
from distutils.core import setup
from setuptools import find_packages
from intervene import __version__ as VERSION


CLASSIFIERS = [
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'Topic :: Software Development :: Libraries :: Python Modules',
]

install_requires = [
    'pybedtools',
    'matplotlib',
    'pandas',
    'numpy',
    'scipy',
]

#def readme():
#    with open('README.rst') as f:
#        return f.read()

def readme(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="intervene",
    description="A tool for intersection and visualization of multiple gene or genomic region sets",
    version=VERSION,
    author="Aziz Khan",
    license='MIT',
    platforms='linux/unix',
    author_email="azez.khan@gmail.com",
    url="https://github.com/asntech/intervene",
    long_description=readme("README.rst"),
    package_dir={'intervene': 'intervene'},

    packages=['intervene',
        'intervene.modules',
        'intervene.modules.pairwise',
        'intervene.modules.venn',
        'intervene.modules.upset',
        'intervene.example_data',
        'intervene.example_data.dbSUPER_mm9',
        'intervene.example_data.ENCODE_hESC',
        'intervene.example_data.Gene_list'],

    scripts=['intervene/intervene',
                   ],
    package_data={'intervene': ['example_data/dbSUPER_mm9/*.bed', 'example_data/ENCODE_hESC/*.bed','example_data/Gene_list/*.txt',]},
    #package_data={'intervene': ['example_data/*']},
    include_package_data=True,
    install_requires = install_requires,
    classifiers=CLASSIFIERS,


)



# coding: utf-8

"""
InterVene: a tool for intersection and visualization of multiple genomic region sets
Created on January 10, 2017
@author: <Aziz Khan>aziz.khan@ncmm.uio.no
"""
import os
import sys
import collections
import time
import os.path as op
from pybedtools import BedTool, chromsizes_to_file, chromsizes
from matplotlib import colors, rc
import numpy as np
import pandas as pd
import pylab as pl
import scipy.cluster.hierarchy as sch
import string
from matplotlib import gridspec


def get_name(fname):
    return op.splitext(op.basename(fname))[0]

def actual_intersection(a, b):
    return len(a.intersect(b, u=True))

def jaccard_of_a(a, b):
    return a.jaccard(b,u=True)['jaccard']

#Calculate the fisher 
def fisher_of_a(a, b, genome):
    return a.fisher(b,genome=genome).two_tail

#Calculate the reldist 
def reldist_of_a(a, b):
    #return mean(a.reldist(b)['reldist'])
    return float(sum(a.reldist(b)['reldist'])) / max(len(a.reldist(b)['reldist']), 1)

def frac_of_a(a, b):
    len_a = float(len(a))
    return len(a.intersect(b, u=True)) / len_a

def enrichment_score(a, b, genome_fn, iterations=1000, processes=1):
    results = a.randomstats(b, new=True, genome_fn=genome_fn, iterations=iterations, processes=processes)
    return (results['actual'] + 1) / (results['median randomized'] + 1)

def create_matrix(beds, func, verbose=False, sort_bed=False, **kwoptions):
    nfiles = len(beds)
    total = nfiles ** 2
    i = 0
    bed_sizes = []
    bed_names = []

    matrix = collections.defaultdict(dict)
    for fa in beds:
        a = BedTool(fa)
        if sort_bed:
            a = a.sort()
        for fb in beds:
            i += 1
            b = BedTool(fb)
            if sort_bed:
                b = b.sort()
            if verbose:
                sys.stderr.write(
                        '%(i)s of %(total)s: %(fa)s + %(fb)s\n' % locals())
                sys.stderr.flush()

            matrix[get_name(fa)][get_name(fb)] = func(a, b, **kwoptions)

        bed_names.append(get_name(fa))
        bed_sizes.append(len(a))
    return matrix, bed_names, bed_sizes

def create_list_matrix(lists, verbose=False):
    nfiles = len(lists)
    total = nfiles ** 2
    i = 0
    list_sizes = []
    list_names = []

    matrix = collections.defaultdict(dict)
    for la in lists:
        with open(la) as f:
            a = f.read().splitlines()
        f.close()
        a = set(a)
        for lb in lists:
            i += 1
            with open(lb) as f:
                b = f.read().splitlines()
            f.close()
            if verbose:
                sys.stderr.write(
                        '%(i)s of %(total)s: %(fa)s + %(fb)s\n' % locals())
                sys.stderr.flush()

            matrix[get_name(la)][get_name(lb)] = len(a.intersection(b))

        list_names.append(get_name(la))
        list_sizes.append(len(a))
    return matrix, list_names, list_sizes

def barplot(series, matrix, outfile, options, max_size=1):
    """Create a bar plot and place the lower triangle of a heatmap directly
    adjacent so that the bases of the bars line up with the diagonal of the
    heatmap. Thanks to Kamil Slowikowski for this code https://gist.github.com/slowkow/5797728

    Parameters
    ----------
    series : pandas.Series
        The bar heights and labels.
    matrix : pandas.DataFrame
        A matrix where each column corresponds to a bar in the bar plot.
    outfile : str
        Full path to the output file.
    figsize : (width, height)
    fontsize : float
    title : str
    """
    # Create a figure.
    fig = pl.figure(figsize=options.figsize)
    gs = gridspec.GridSpec(1, 2, width_ratios=[6, 1]) 

    # Axes for the heatmap triangle.
    ax = fig.add_subplot(gs[0], frame_on=False, aspect=2.0)
    #ax = fig.add_subplot(121, frame_on=False, aspect=2.0)
    
    # Get the heatmap triangle's axes and the order of the clustered samples.
    cax, order = heatmap_triangle(matrix, ax, options)

    # Adjust spacing between the heatmap triangle and the barplot.
    #fig.subplots_adjust(wspace=-0.25, hspace=0, left=0, right=1)
    #right=1.1 
    fig.subplots_adjust(wspace=0, hspace=0, left=0, right=options.space)

    # Axes for the barplot.
    #ax = fig.add_subplot(122, frame_on=False)
    ax = fig.add_subplot(gs[1], frame_on=False)

    # Put gridlines beneath the bars.
    ax.set_axisbelow(True)

    # Order the bars by the clustering.
    series = series.ix[order]

    namelen = 15
    # Shorten lengthy names.
    series.index = [shorten(x, namelen) for x in series.index]

    ax = series.plot(ax=ax, kind='barh', title=options.title, linewidth=0,
                     grid=False, color=options.barcolor)

    # Set the font size for the y-axis labels.
    ax.tick_params(axis='y', which='major', labelsize=options.fontsize)

    # Grid lines.
    ax.grid(b=False, which='major', axis='both', alpha=0.1)

    # Tick marks for the x-axis. max(list_size)
    ax.set_xticks((max_size,1))

    # Put the y-axis marks on the right.
    ax.yaxis.tick_right()
    ax.yaxis.set_label_position('right')

    # Adjust tick length.
    ax.tick_params(length=0, axis='x', labelsize=7)
    ax.tick_params(length=0, axis='y')

    # Labels.
    ax.set_xlabel(options.blabel, size=8)
    ax.set_ylabel('')

    fig.tight_layout()

    #fig.show()
    # Save.
    fig.savefig(outfile, bbox_inches='tight', dpi=options.dpi)

def shorten(x, n=48):
    x = str(x)
    if len(x) > n:
        return x[:int(n/2)] + '..' + x[-int(n/2):]
    return x

def heatmap_triangle(dataframe, axes, options):
    """Create a heatmap of the lower triangle of a pairwise correlation
    matrix of all pairs of columns in the given dataframe. The heatmap
    triangle is rotated 45 degrees clockwise and drawn on the given axes.
    Thanks to Kamil Slowikowski for sharing this code https://gist.github.com/slowkow/5797728

    Parameters
    ----------
    dataframe : pandas.DataFrame
    axes : matplotlib.axes.Axes
    """
    N = dataframe.shape[1]
    D = dataframe

    tri_type = options.triangle
    
    #D = dataframe.corr(method='pearson')

    # UPGMA clustering, but other methods are also available.
    Z = sch.linkage(D, method='average')
    R = sch.dendrogram(Z, no_plot=True)
    cluster_order = R['leaves']
    D = D.ix[cluster_order, cluster_order]

    # Get the lower triangle of the matrix. 
    #C = np.tril(D)
    # Mask the lower triangle.
    #L = np.ma.masked_array(L, L == 0)
    
    if tri_type == "upper":
        # Get the upper triangle of the matrix.
        C = np.triu(D)
        C = np.transpose(C)
    elif tri_type == "full":
        C = D
    # Get the lower triangle of the matrix. 
    else:
        C = np.tril(D)
    
    # Mask the upper triangle.
    C = np.ma.masked_array(C, C == 0)

    #diagonal_val = 1
    # Set the diagonal to zero.
    #for i in range(N):
    #    C[i, i] = diagonal_val

    # Transformation matrix for rotating the heatmap.
    A = np.array([(y, x) for x in range(N, -1, -1) for y in range(N + 1)])
    t = np.array([[0.5, 1], [0.5, -1]])
    A = np.dot(A, t)

    if options.compute == 'reldist':
        min_val = 0.0
        max_val = 0.5

    elif options.compute == 'count':
        min_val = 0.0
        max_val = np.round(np.amax(C), decimals=1)
    else:
        min_val = np.round(np.amin(C), decimals=1)
        max_val = np.round(np.amax(C), decimals=1)

    #if min_val == 0:
    #   min_val = -1
    # -1.0 correlation is blue, 0.0 is white, 1.0 is red.
    # 1.0 correlation is blue, 0.0 is white, 1.0 is red.
    cmap = pl.cm.RdBu_r
    norm = colors.BoundaryNorm(np.linspace(min_val, max_val, 20), cmap.N)

    # This MUST be before the call to pl.pcolormesh() to align properly.
    axes.set_xticks([])
    axes.set_yticks([])

    # Plot the correlation heatmap triangle.
    if tri_type == "full":
        X = A[:, 1].reshape(N + 1, N + 1)
        Y = A[:, 1].reshape(N + 1, N + 1)
        caxes = pl.pcolormesh(X, Y, np.flipud(C), axes=axes, cmap=cmap, norm=norm)
    else:
        X = A[:, 1].reshape(N + 1, N + 1)
        Y = A[:, 0].reshape(N + 1, N + 1)
        caxes = pl.pcolormesh(X, Y, np.flipud(C), axes=axes, cmap=cmap, norm=norm)

    # Remove the ticks and reset the x limit.
    axes.set_xlim(right=1)
    #axes.labelsize = "small"
    #axes.tick_params(labelsize=6)
    if options.compute == 'count':
        ticks = np.linspace(min_val, max_val, 3)
    elif options.compute == 'reldist':
        ticks = np.linspace(min_val, max_val, 3)
    else:
        ticks = np.linspace(min_val, 1.0, 3)
    # Add a colorbar below the heatmap triangle.
    cb = pl.colorbar(caxes, ax=axes, orientation='horizontal', shrink=0.5825,
                     fraction=0.02, pad=0, ticks=ticks,
                     use_gridspec=True)
    cb.set_label(options.hlabel)

    return caxes, D.index


def create_r_script(matrix_file, options, max_size=1):
    """
    
    It create an Rscript for pairwise heatmap plot for the genomic regions.

    """

    if not options.figsize:
        options.figsize = (8,8)

    if options.compute == 'reldist':
        min_val = 0.0
        max_val = 0.5

    elif options.compute == 'count':
        min_val = 0.0
        max_val = max_size
    else:
        min_val = 0.0
        max_val = 1.0

    if options.diagonal:
        diag = 'diag=TRUE'
    else:
        diag = 'diag=FALSE'
    
    script_file =  options.output+'/'+str(options.project)+'_'+options.command+'_'+options.compute+'.R'
    temp_f = open(script_file, 'w')
    output_name = options.output+'/'+str(options.project)+'_'+options.command+'_'+options.compute+'.'+options.figtype

    temp_f.write('#!/usr/bin/env Rscript'+"\n")
    temp_f.write('if (suppressMessages(!require("corrplot"))) suppressMessages(install.packages("corrplot", repos="http://cran.us.r-project.org"))\n')
    temp_f.write('library("corrplot")\n')
    if options.figtype == 'ps':
        temp_f.write('if (suppressMessages(!require("Cairo"))) suppressMessages(install.packages("Cairo", repos="http://cran.us.r-project.org"))\n')
        temp_f.write('library("Cairo")\n')
    
    if options.figtype == 'pdf' or options.figtype == 'svg':
        temp_f.write(options.figtype+'("'+output_name+'", width='+str(options.figsize[0])+', height='+str(options.figsize[1])+')'+'\n')
    
    elif options.figtype == 'ps':
        temp_f.write('cairo_ps("'+output_name+'", width='+str(options.figsize[0])+', height='+str(options.figsize[1])+')'+'\n')
    else:
        temp_f.write(options.figtype+'("'+output_name+'", width='+str(options.dpi*options.figsize[0])+', height='+str(options.dpi*options.figsize[1])+', res='+str(options.dpi)+')\n')
     
    temp_f.write("\n")

    #cmd = 'heatmap_intervene.R %s %s %s %s %s %s %s' % (matrix_file,,options.compute, output_name,, , options.dpi)
    #cl.lim= c('+str(min_val)+','+str(max_val)+'), 
    temp_f.write('intersection_matrix <- as.matrix(read.table("'+matrix_file+'"))\n')
    temp_f.write('corrplot(intersection_matrix, method ="'+options.htype+'", title="'+str(options.title)+'", tl.col="black", tl.cex=0.8, is.corr = FALSE, '+diag+', addrect=1, mar=c(0,0,2,1), rect.col = "black")\n')
    temp_f.write('invisible(dev.off())\n')

    cmd = temp_f.name
    temp_f.close()

    if options.scriptonly == False:
        os.system('chmod +x '+cmd)
        os.system(cmd)
        print('\nYou are done! Please check your results @ '+options.output+'. \nThank you for using Intervene!\n')
        sys.exit(0)
    else:
        print('\nYou are done! Please check your UpSet plot script and Shiny App input @ '+options.output+'. \nThank you for using Intervene!\n')
        sys.exit(0)
        
def pairwise_intersection(options):

    '''
    if options.enrichment:
        FUNC = enrichment_score
        genome_fn = chromsizes_to_file(chromsizes(options.genome))
        kwoptions = dict(genome_fn=genome_fn, iterations=options.iterations,
                processes=options.processes)
    '''

    if options.type == 'genomic':
        if options.compute == "frac":
            FUNC = frac_of_a
            kwoptions = {}

        elif options.compute == 'jaccard':
            FUNC = jaccard_of_a
            kwoptions = {}

        elif options.compute == 'fisher':
            FUNC = fisher_of_a
            kwoptions = dict(genome=options.genome)

        elif options.compute == 'reldist':
            FUNC = reldist_of_a
            kwoptions = {}

        else:
            FUNC = actual_intersection
            kwoptions = {}

        matrix, bed_names, bed_sizes = create_matrix(beds=options.input, func=FUNC, verbose=False, sort_bed=options.sort, **kwoptions)
    else:
        matrix, bed_names, bed_sizes = create_list_matrix(lists=options.input, verbose=False)

    nfiles = len(options.input)

    script_file =  options.output+'/'+str(options.project)+'_'+options.command+'_'+options.compute+'.R'

    #if options.verbose:
    #    sys.stderr.write('Time to construct %s x %s matrix: %.1fs' \
    #            % (nfiles, nfiles, (t1 - t0)) + '\n')
    keys = sorted(matrix.keys())
    
    matrix_file =  options.output+'/'+str(options.project)+'_'+options.command+'_'+options.compute+'_matrix.txt'

    f = open(matrix_file, 'w')

    #if options.stdout:
    #sys.stdout.write("\t" + "\t".join(keys) + '\n')
    f.write("\t" + "\t".join(keys) + '\n')
    for k in keys:
        #sys.stdout.write(k)
        f.write(k)
        for j in keys:
            #sys.stdout.write('\t' + str(matrix[k][j]))
            f.write('\t' + str(matrix[k][j]))
        #sys.stdout.write('\n')
        f.write('\n')
    f.close()

    if options.htype == 'tribar':
        rc("font", family="serif")
        ncols = nfiles
        matrix = pd.read_table(matrix_file,index_col=0, delim_whitespace=True)

        labels = list(matrix.columns.values)
        labels = bed_names
        series = pd.Series(bed_sizes, index=labels)
        #Set heatmap label
        if options.compute == 'count':
            options.hlabel = 'Number of overlaps'
        if options.compute == 'frac':
            options.hlabel = 'Fraction of overlap'
        if options.compute == 'jaccard':
            options.hlabel = 'Jaccard statistic'
        if options.compute == 'reldist':
            options.hlabel = 'Dist. of relative distance'
        if options.compute == 'fisher':
            options.hlabel = "Fisher's p-value"

        #options.title = "Pairwise intersection"
        #options.figsize=(8, 6)
        #series = pd.Series(np.random.random(ncols) * 2.0, index=labels)
        #df = pd.read_csv('data.csv',index_col=0, delim_whitespace=True)
        #matrix = pd.DataFrame(np.random.random((nrows, ncols)), columns=labels)
        outfile =  options.output+'/'+str(options.project)+'_'+options.command+'_'+options.compute+'.'+options.figtype

        barplot(series, matrix, outfile, options, max_size=max(bed_sizes))

        print('\nYou are done! Please check your results @ '+options.output+'. \nThank you for using Intervene!\n')
        
    else:
        #print("Please check the matrix file "+matrix_file)
        create_r_script(matrix_file, options, max_size=max(bed_sizes))
        #cmd = 'heatmap_intervene.R %s %s %s %s %s %s %s' % (matrix_file,options.htype,options.compute, output_name,options.figtype, str(options.title), options.dpi)
        #os.system(cmd)
        #print('\nYou are done! Please check your results @ '+options.output+'. \nThank you for using Intervene!\n')